@@ -27,12 +27,10 @@
 
 static void _no_fin_locking(void)
 {
-	return;
 }
 
 static void _no_reset_locking(void)
 {
-	return;
 }
 
 static int _no_lock_resource(struct cmd_context *cmd, const char *resource,
@@ -71,10 +69,13 @@ static int _readonly_lock_resource(struct cmd_context *cmd,
 				   uint32_t flags)
 {
 	if ((flags & LCK_TYPE_MASK) == LCK_WRITE &&
-	    (flags & LCK_SCOPE_MASK) == LCK_VG) {
+	    (flags & LCK_SCOPE_MASK) == LCK_VG &&
+	    !(flags & LCK_CACHE) &&
+	    strcmp(resource, VG_GLOBAL)) {
 		log_error("Write locks are prohibited with --ignorelockingfailure.");
 		return 0;
 	}
+
 	return _no_lock_resource(cmd, resource, flags);
 }
 
