@@ -13,9 +13,10 @@
  */
 
 #include "lib.h"
-#include "lvm.h"
+#include "lvm2app.h"
 #include "toolcontext.h"
 #include "locking.h"
+#include "lvm-version.h"
 
 const char *lvm_library_get_version(void)
 {
