@@ -14,7 +14,9 @@ which sfdisk || exit 200
 which perl || exit 200
 which awk || exit 200
 which cut || exit 200
-test -f /proc/mdstat || exit 200
+
+test -f /proc/mdstat && grep -q raid0 /proc/mdstat || \
+modprobe raid0 || exit 200
 
 . ./test-utils.sh
 
@@ -49,14 +51,14 @@ trap 'aux cleanup_md' EXIT # cleanup this MD device at the end of the test
 
 # Test alignment of PV on MD without any MD-aware or topology-aware detection
 # - should treat $mddev just like any other block device
-pv_align="192.00K"
+pv_align="192.00k"
 pvcreate --metadatasize 128k \
     --config 'devices {md_chunk_alignment=0 data_alignment_detection=0 data_alignment_offset_detection=0}' \
     $mddev
 check_pv_field_ $mddev pe_start $pv_align
 
 # Test md_chunk_alignment independent of topology-aware detection
-pv_align="256.00K"
+pv_align="256.00k"
 pvcreate --metadatasize 128k \
     --config 'devices {data_alignment_detection=0 data_alignment_offset_detection=0}' \
     $mddev
@@ -66,15 +68,16 @@ check_pv_field_ $mddev pe_start $pv_align
 linux_minor=$(echo `uname -r` | cut -d'.' -f3 | cut -d'-' -f1)
 
 # Test newer topology-aware alignment detection
-if [ $linux_minor -gt 31 ]; then
-    pv_align="256.00K"
+# - first added to 2.6.31 but not "reliable" until 2.6.33
+if [ $linux_minor -ge 33 ]; then
+    pv_align="256.00k"
     pvcreate --metadatasize 128k \
 	--config 'devices { md_chunk_alignment=0 }' $mddev
     check_pv_field_ $mddev pe_start $pv_align
 fi
 
 # partition MD array directly, depends on blkext in Linux >= 2.6.28
-if [ $linux_minor -gt 27 ]; then
+if [ $linux_minor -ge 28 ]; then
     # create one partition
     sfdisk $mddev <<EOF
 ,,83
@@ -95,17 +98,17 @@ EOF
 
     # Checking for 'alignment_offset' in sysfs implies Linux >= 2.6.31
     sysfs_alignment_offset=/sys/dev/block/${mddev_maj_min}/${base_mddev_p}/alignment_offset
-    [ -f $sysfs_alignment_offset ] && \
+    [ -f $sysfs_alignment_offset -a $linux_minor -ge 33 ] && \
 	alignment_offset=`cat $sysfs_alignment_offset` || \
 	alignment_offset=0
 
     if [ "$alignment_offset" = "512" ]; then
-	pv_align="256.50K"
+	pv_align="256.50k"
 	pvcreate --metadatasize 128k $mddev_p
 	check_pv_field_ $mddev_p pe_start $pv_align
 	pvremove $mddev_p
     elif [ "$alignment_offset" = "2048" ]; then
-	pv_align="258.00K"
+	pv_align="258.00k"
 	pvcreate --metadatasize 128k $mddev_p
 	check_pv_field_ $mddev_p pe_start $pv_align
 	pvremove $mddev_p
